<?php

namespace Drupal\change_labels\EventSubscriber;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field_event_dispatcher\Event\Field\FieldWidgetThirdPartySettingsFormEvent;
use Drupal\field_event_dispatcher\Event\Field\WidgetCompleteFormAlterEvent;
use Drupal\field_event_dispatcher\FieldHookEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Change labels event subscriber.
 */
class ChangeFieldLabelSubscriber implements EventSubscriberInterface, TrustedCallbackInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      FieldHookEvents::FIELD_WIDGET_THIRD_PARTY_SETTINGS_FORM => ['addSettings'],
      FieldHookEvents::WIDGET_COMPLETE_FORM_ALTER => ['alterForm'],
    ];
  }

  /**
   * Event handler for the third party settings of a widget.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\FieldWidgetThirdPartySettingsFormEvent $event
   *   Triggering event.
   */
  public function addSettings(FieldWidgetThirdPartySettingsFormEvent $event) {
    $element = [];
    $element['field_label_overwrite'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Overwrite field label'),
      '#description' => $this->t('Enter &ltnolabel&gt; to remove the label.'),
      '#default_value' => $event->getPlugin()->getThirdPartySetting('change_labels', 'field_label_overwrite'),
    ];
    $event->addElements('change_labels', $element);
  }

  /**
   * Event handler for altering a widget.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\WidgetCompleteFormAlterEvent $event
   *   Triggering event.
   */
  public function alterForm(WidgetCompleteFormAlterEvent $event) {
    $setting = $event->getContext()['widget']->getThirdPartySetting('change_labels', 'field_label_overwrite');
    if ($setting) {
      $elements = &$event->getElements();
      foreach (Element::children($elements) as $child) {
        if ($setting === '<nolabel>') {
          $elements[$child]['#title_display'] = 'invisible';
          $elements[$child]['#process'][] = [
            static::class,
            'processChangeFieldLabel',
          ];
          continue;
        }
        $elements[$child]['#title'] = $setting;
      }
    }
  }

  /**
   * Process callback to add a pre_render after the ones added by the theme.
   */
  public static function processChangeFieldLabel($element, FormStateInterface $form_state, $form) {
    $element['#pre_render'][] = [
      static::class,
      'overwriteLabel',
    ];
    return $element;
  }

  /**
   * Process callback to hide label when there is a details element.
   *
   * For example single file widgets.
   */
  public static function overwriteLabel($element) {
    if (isset($element['#theme_wrappers']['details'])) {
      $element['#theme_wrappers']['details']['#summary_attributes']['class'][] = 'visually-hidden';
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return [
      'overwriteLabel',
    ];
  }

}
